@@ -2,7 +2,7 @@
 # encoding: utf-8
 
 __author__ = "ChenyangGao <https://chenyanggao.github.io>"
-__version__ = (0, 1)
+__version__ = (0, 1, 2)
 __all__ = [
     "Buffer", "SupportsRead", "SupportsReadinto", 
     "SupportsWrite", "SupportsSeek", 
@@ -12,16 +12,17 @@ __all__ = [
     "bytes_iter_to_reader", "bytes_iter_to_async_reader", 
     "bytes_to_chunk_iter", "bytes_to_chunk_async_iter", 
     "bytes_ensure_part_iter", "bytes_ensure_part_async_iter", 
+    "progress_bytes_iter", "progress_bytes_async_iter", 
 ]
 
 from asyncio import to_thread, Lock as AsyncLock
 from collections.abc import Awaitable, AsyncIterable, AsyncIterator, Callable, Iterable, Iterator
 from functools import update_wrapper
-from inspect import isawaitable, iscoroutinefunction
+from inspect import isawaitable, iscoroutinefunction, isasyncgen, isgenerator
 from itertools import chain
 from shutil import COPY_BUFSIZE # type: ignore
 from threading import Lock
-from typing import runtime_checkable, Any, Protocol, TypeVar
+from typing import cast, runtime_checkable, Any, ParamSpec, Protocol, TypeVar
 
 try:
     from collections.abc import Buffer # type: ignore
@@ -59,10 +60,10 @@ except ImportError:
     Buffer.register(memoryview)
     Buffer.register(array)
 
-
 from asynctools import async_chain, ensure_async, ensure_aiter
 
 
+Args = ParamSpec("Args")
 _T_co = TypeVar("_T_co", covariant=True)
 _T_contra = TypeVar("_T_contra", contravariant=True)
 
@@ -655,3 +656,83 @@ async def bytes_ensure_part_async_iter(
             else:
                 n = partsize
 
+
+def progress_bytes_iter(
+    it: Iterable[Buffer] | Callable[[], Buffer], 
+    make_progress: None | Callable[Args, Any] = None, 
+    /, 
+    *args: Args.args, 
+    **kwds: Args.kwargs, 
+) -> Iterator[Buffer]:
+    update_progress: None | Callable = None
+    close_progress: None | Callable = None
+    if callable(it):
+        it = iter(it, b"")
+    if callable(make_progress):
+        progress = make_progress(*args, **kwds)
+        if isgenerator(progress):
+            next(progress)
+            update_progress = progress.send
+            close_progress = progress.close
+        else:
+            update_progress = progress
+            close_progress = getattr(progress, "close", None)
+    try:
+        if callable(update_progress):
+            for chunk in it:
+                yield chunk
+                update_progress(len(chunk))
+        else:
+            for chunk in it:
+                yield chunk
+    finally:
+        if callable(close_progress):
+            close_progress()
+
+
+async def progress_bytes_async_iter(
+    it: Iterable[Buffer] | AsyncIterable[Buffer] | Callable[[], Buffer] | Callable[[], Awaitable[Buffer]], 
+    make_progress: None | Callable[Args, Any] = None, 
+    /, 
+    *args: Args.args, 
+    **kwds: Args.kwargs, 
+) -> AsyncIterator[Buffer]:
+    update_progress: None | Callable = None
+    close_progress: None | Callable = None
+    if callable(it):
+        async def wrapiter(it) -> Buffer:
+            while True:
+                chunk = it()
+                if isawaitable(chunk):
+                    chunk = await chunk
+                if not chunk:
+                    break
+                yield chunk
+        it = cast(AsyncIterable[Buffer], wrapiter(it))
+    if callable(make_progress):
+        progress = make_progress(*args, **kwds)
+        if isgenerator(progress):
+            await ensure_async(next)(progress)
+            update_progress = progress.send
+            close_progress = progress.close
+        elif isasyncgen(progress):
+            await anext(progress)
+            update_progress = progress.asend
+            close_progress = progress.aclose
+        else:
+            update_progress = progress
+            close_progress = getattr(progress, "close", None)
+    try:
+        it = ensure_aiter(it)
+        if callable(update_progress):
+            update_progress = ensure_async(update_progress)
+            async for chunk in it:
+                yield chunk
+                await update_progress(len(chunk))
+        else:
+            async for chunk in it:
+                yield chunk
+    finally:
+        if callable(close_progress):
+            await ensure_async(close_progress)()
+
